// Shell Driver
// shelldriver.cpp
// By Bryce Bigwood 1998
// public domain

// include some posix stuff
#include <unistd.be.h>

static	int	fd1[2], fd2[2];
static	char	readBuffer[1024];
static	char	*writeString = "ls / -a -i -l\n";
	
static int32	thread_read_proc(void *arg)
{
	// in this sample, we will just loop forever, waiting for inooming text from sh's standard out
	while (1)
	{
		// read() blocks
		int n = read(fd2[0], readBuffer, 1024);
		if (n > 0)
		{
			// note that the standard out of this team is
			// independent of the standard out of the loaded shell,
			// (which is a different team)
			printf("%s", readBuffer);
		}
	}
	
	return B_NO_ERROR;
}

int main(int argc, char *argv[])
{
	// load shellstub
	extern char **environ;

	char	**arg_v;
	int32	arg_c = 0;
	thread_id	exec_thread = 0;
	int32	return_value = 0;
	
	// first, create the pipes
	if (pipe(fd1) < 0 || pipe(fd2) < 0)
		return -1;
		
	//printf("file descriptors: fd1[0] = %d, fd1[1] = %d, fd2[0] = %d, fd2[1] = %d\n", fd1[0], fd1[1], fd2[0], fd2[1]);
	
	arg_c = 6;
	arg_v = (char**)malloc(sizeof(char *) * (arg_c + 1));
	
	char fdbuffer[64];
	
	BPath path(argv[0]);
	BPath parentPath;
	path.GetParent(&parentPath);
	BPath stubPath(parentPath.Path(), "ForkStub", true);
	//printf("path: %s\n", path.Path());
	//printf("parent path: %s\n", parentPath.Path());
	//printf("stub path: %s\n", stubPath.Path());
	
	// binary to load (i.e. ShellStub)
	arg_v[0] = strdup(stubPath.Path());
	// binary that above-loaded-binary is supposed to load (e.g. /bin/sh)
	arg_v[1] = strdup("/bin/sh");
	// file descriptors
	sprintf(fdbuffer, "%d", fd1[0]);
	arg_v[2] = (char*)malloc(strlen(fdbuffer));
	strcpy(arg_v[2], fdbuffer);
	sprintf(fdbuffer, "%d", fd1[1]);
	arg_v[3] = (char*)malloc(strlen(fdbuffer));
	strcpy(arg_v[3], fdbuffer);
	sprintf(fdbuffer, "%d", fd2[0]);
	arg_v[4] = (char*)malloc(strlen(fdbuffer));
	strcpy(arg_v[4], fdbuffer);
	sprintf(fdbuffer, "%d", fd2[1]);
	arg_v[5] = (char*)malloc(strlen(fdbuffer));
	strcpy(arg_v[5], fdbuffer);
	//printf("%s\n", arg_v[2]);
	//printf("%s\n", arg_v[3]);
	//printf("%s\n", arg_v[4]);
	//printf("%s\n", arg_v[5]);
	arg_v[6] = NULL;
	
	exec_thread = load_image(arg_c, arg_v, environ);
	
	// close read end of pipe 1
	close(fd1[0]);
	// close write end of pipe 2
	close(fd2[1]);
	
	status_t status = resume_thread(exec_thread);
	if (status < B_NO_ERROR)
		return -1;
	
	// now interact
	thread_id read_thread = spawn_thread(thread_read_proc, "Read Thread", B_NORMAL_PRIORITY, NULL);
	if (read_thread < B_NO_ERROR)
		return -1;
		
	if (write(fd1[1], writeString, strlen(writeString)) != strlen(writeString))
		return -1;
	
	status_t exit_value = B_NO_ERROR;
	status = wait_for_thread(read_thread, &exit_value);
	
	return exit_value;
}

